#pragma once

#include <a8/reflect.h>
#include <a8/stringlist.h>
#include <a8/xobject.h>

namespace a8
{
    namespace mysql
    {
        class Connection;
        class Query;
    }
}

namespace f8
{

    template <typename T>
    bool ReadJsonMetaFile(const std::string& filename, std::vector<T*>& meta_list)
    {
        T dummy;
        auto cls = dummy.GetClass();

        a8::XObject xobj_arr;
        xobj_arr.ReadFromFile(filename);
        for (int i = 0; i < xobj_arr.Size(); ++i) {
            T* obj = new T();
            a8::reflect::Field* fields = cls->GetDeclaredFields();
            for (int ii = 0; ii < cls->FieldNum(); ++ii) {
                a8::reflect::Field& field = fields[ii];
                if (field.subtype != a8::reflect::EST_SIMPLE) {
                    abort();
                }
                auto xobj = xobj_arr.At(i);
                void* p = (char*)obj + field.offset;
                if (xobj->HasKey(field.field_name)) {
                    switch (field.type) {
                    case a8::reflect::ET_INT32:
                        {
                            *((int*)p) = xobj->At(field.field_name)->AsXValue();
                        }
                        break;
                    case a8::reflect::ET_UINT32:
                        {
                            *((unsigned int*)p) = xobj->At(field.field_name)->AsXValue();
                        }
                        break;
                    case a8::reflect::ET_INT64:
                        {
                            *((long long*)p) = xobj->At(field.field_name)->AsXValue();
                        }
                        break;
                    case a8::reflect::ET_FLOAT:
                        {
                            *((float*)p) = xobj->At(field.field_name)->AsXValue().GetDouble();
                        }
                        break;
                    case a8::reflect::ET_DOUBLE:
                        {
                            *((double*)p) = xobj->At(field.field_name)->AsXValue();
                        }
                        break;
                    case a8::reflect::ET_STRING:
                        {
                            *((std::string*)p) = xobj->At(field.field_name)->AsXValue().GetString();
                        }
                        break;
                    default:
                        {
                            abort();
                        }
                    }
                    obj->__flags__.set(ii);
                }
            }
            meta_list.push_back(obj);
        }
        return true;
    }

    std::string PbToJson(const google::protobuf::Message* msg);

    void InitMysqlConnection(a8::mysql::Query* query);
    void CheckMysqlConnection(a8::mysql::Connection* conn, a8::mysql::Query* query,
                              std::string dbhost, int port,  std::string dbuser, std::string dbpasswd, std::string gamedb);

    bool CheckRegisterTimeInSessionId(const std::string& accountid, const std::string& sessionid);
    time_t ExtractRegisterTimeFromSessionId(const std::string& sessionid);
    bool IsValidSessionId(const std::string& accountid, const std::string& sessionid);
    int ExtractGameIdFromAccountId(const std::string& accountid);
    int ExtractChannelIdFromAccountId(const std::string& accountid);

    bool IsOnlineEnv();
    bool IsTestEnv();
    bool IsValidNormalConfig(a8::XObject& conf, std::vector<std::string> fields);

}
